package com.persagy.apm.energy.report.monthly.outline.service.builder.impl;

import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.AddReportBusinessDetailDTO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.CostVO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.PowerVO;
import com.persagy.apm.energy.report.monthly.detail.business.service.IReportBusinessDetailService;
import com.persagy.apm.energy.report.monthly.outline.constants.BusinessReportParagraphs;
import com.persagy.apm.energy.report.monthly.outline.constants.enums.ReportStateEnum;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.*;
import com.persagy.apm.energy.report.monthly.outline.service.builder.IBusinessReportBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.List;

/**
 * 商业项目报告建造者
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class BusinessReportBuilder implements IBusinessReportBuilder {
    @Autowired
    private IReportBusinessDetailService businessDetailService;
    @Autowired
    private IBusinessReportPowerInfoService businessReportPowerInfoService;
    @Autowired
    private IBusinessReportCostInfoService businessReportCostInfoService;
    @Autowired
    private IBusinessReportPlatformInfoService businessReportPlatformInfoService;
    @Autowired
    private IReportOutlineService reportOutlineService;

    /**
     * 生成报告
     *
     * @param reportOutline 报告outline对象
     * @author lixing
     * @version V1.0 2021/5/24 4:09 下午
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public void generateReport(ReportOutline reportOutline) {
        try {
            // 删除之前关联的报告信息
            deletePreviousReportDetail(reportOutline.getDetailId());

            AddReportBusinessDetailDTO addReportBusinessDetailDTO = new AddReportBusinessDetailDTO();
            // 获取项目用电量信息
            addReportBusinessDetailDTO.setOpenPower(getOpenPowerInfo(reportOutline));
            addReportBusinessDetailDTO.setComparePower(getComparePowerInfo(reportOutline));
            // 获取项目费用信息
            addReportBusinessDetailDTO.setOpenCost(getOpenCostInfo(reportOutline));
            addReportBusinessDetailDTO.setCompareCost(getCompareCostInfo(reportOutline));

            // 获取项目平台运维信息
            addReportBusinessDetailDTO.setPlatform(getPlatforms(reportOutline));

            String detailId = businessDetailService.createReportBusinessDetail(addReportBusinessDetailDTO);
            reportOutline.setDetailId(detailId);
            updateReportOutlineInfoWhenSuccess(reportOutline);
        } catch (Exception e) {
            log.error("生成报告失败", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            updateReportOutlineInfoWhenFailure(reportOutline);
        }
    }

    @Override
    public void deletePreviousReportDetail(String detailId) {
        businessDetailService.deleteReportBusinessDetail(detailId);
    }

    @Override
    public PowerVO getOpenPowerInfo(ReportOutline reportOutline) {
        List<String> powerGroupIdList = businessDetailService.getGroupsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_POWER);
        List<String> powerFunctionIdList = businessDetailService.getFunctionsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_POWER);
        return businessReportPowerInfoService.getPowerInfo(reportOutline, powerGroupIdList, powerFunctionIdList);
    }

    @Override
    public PowerVO getComparePowerInfo(ReportOutline reportOutline) {
        List<String> powerGroupIdList = businessDetailService.getGroupsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_POWER);
        List<String> powerFunctionIdList = businessDetailService.getFunctionsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_POWER);
        return businessReportPowerInfoService.getComparePowerInfo(reportOutline, powerGroupIdList, powerFunctionIdList);
    }

    @Override
    public CostVO getOpenCostInfo(ReportOutline reportOutline) {
        List<String> costGroupIdList = businessDetailService.getGroupsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_COST);
        List<String> costFunctionIdList = businessDetailService.getFunctionsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_COST);
        return businessReportCostInfoService.getOpenCost(reportOutline, costGroupIdList, costFunctionIdList);
    }

    @Override
    public CostVO getCompareCostInfo(ReportOutline reportOutline) {
        List<String> costGroupIdList = businessDetailService.getGroupsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_COST);
        List<String> costFunctionIdList = businessDetailService.getFunctionsByParagraph(
                reportOutline.getReportTypeId(), BusinessReportParagraphs.OPEN_COST);
        return businessReportCostInfoService.getCompareCost(reportOutline, costGroupIdList, costFunctionIdList);
    }

    @Override
    public List<Platform> getPlatforms(ReportOutline reportOutline) {
        return businessReportPlatformInfoService.getPlatforms(reportOutline);
    }

    @Override
    public void updateReportOutlineInfoWhenSuccess(ReportOutline reportOutline) {
        reportOutline.setStatus(ReportStateEnum.GENERATED.getType());
        reportOutlineService.updateReportOutline(reportOutline);
    }

    @Override
    public void updateReportOutlineInfoWhenFailure(ReportOutline reportOutline) {
        reportOutline.setStatus(ReportStateEnum.BUILD_FAILED.getType());
        reportOutlineService.updateReportOutline(reportOutline);
    }
}
