package com.persagy.apm.energy.report.monthly.outline.service.impl;

import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.function.service.IFunctionService;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.FunctionGroup;
import com.persagy.apm.energy.report.monthly.config.functiongroup.service.IFunctionGroupService;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.CostItemVO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.CostVO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.GroupInfo;
import com.persagy.apm.energy.report.monthly.functionvalue.service.IFunctionValueService;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.IBusinessReportCostInfoService;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 商业类报告费用service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 7:30 下午
 **/
@Service
public class BusinessReportCostInfoServiceImpl implements IBusinessReportCostInfoService {
    @Autowired
    private IFunctionGroupService functionGroupService;
    @Autowired
    private IFunctionService functionService;
    @Autowired
    private IFunctionValueService functionValueService;
    @Autowired
    private ISaasWebService saasWebService;
    @Autowired
    private IReportOutlineService reportOutlineService;

    @Override
    public CostVO getOpenCost(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList) {

        Date reportMonth = reportOutline.getReportMonth();

        List<String> projectIds = reportOutlineService.queryRelatedProjects(reportOutline);

        return getCostInfoByProjectIds(groupIdList, functionIdList, reportMonth, projectIds);
    }

    @Override
    public CostVO getCompareCost(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList) {

        Date reportMonth = reportOutline.getReportMonth();

        List<String> projectIds = reportOutlineService.queryRelatedProjects(reportOutline);

        List<String> comparableProjects = saasWebService.getComparableProjects(projectIds, reportMonth);

        return getCostInfoByProjectIds(groupIdList, functionIdList, reportMonth, comparableProjects);
    }

    /**
     * 根据项目id获取费用信息
     *
     * @param groupIdList    费用信息点分组列表
     * @param functionIdList 费用信息点列表
     * @param reportMonth    报告月份
     * @param projectIds     项目id列表
     * @return 费用信息
     * @author lixing
     * @version V1.0 2021/5/30 11:41 下午
     */
    private CostVO getCostInfoByProjectIds(List<String> groupIdList, List<String> functionIdList, Date reportMonth, List<String> projectIds) {
        CostVO costVO = new CostVO();
        List<GroupInfo<CostItemVO>> costGroupInfos = getCostGroupInfos(groupIdList, reportMonth, projectIds);
        costVO.setGroups(costGroupInfos);

        List<CostItemVO> costItemInfos = getCostItemInfos(functionIdList, reportMonth, projectIds);
        costVO.setItems(costItemInfos);

        List<CostItemVO> paragraphSums = new ArrayList<>();

        if (!CollectionUtils.isEmpty(costGroupInfos)) {
            paragraphSums.addAll(
                    costGroupInfos.stream().map(GroupInfo::getSummary).collect(Collectors.toList()));
        }
        if (!CollectionUtils.isEmpty(costItemInfos)) {
            paragraphSums.addAll(costItemInfos);
        }
        costVO.setSummary(getCostSum(paragraphSums));
        return costVO;
    }


    @Override
    public List<CostItemVO> getCostItemInfos(List<String> functionIdList, Date reportMonth, List<String> projectIds) {
        List<CostItemVO> costItemVOList = Lists.newArrayList();
        if (CollectionUtils.isEmpty(functionIdList) || CollectionUtils.isEmpty(projectIds)) {
            return costItemVOList;
        }

        for (String functionId : functionIdList) {
            Function function = functionService.queryFunctionDetail(functionId);
            String itemName = functionService.getItemName(function, projectIds.get(0));

            List<CostItemVO> projectCostItems = getCostItemsByProjects(reportMonth, projectIds, function);
            CostItemVO costSumByProjects = getCostSum(projectCostItems);
            costSumByProjects.setName(itemName);
            costItemVOList.add(costSumByProjects);
        }
        return costItemVOList;
    }

    @Override
    public List<CostItemVO> getCostItemsByProjects(Date reportMonth, List<String> projectIds, Function function) {
        List<CostItemVO> projectCostItems = Lists.newArrayList();
        if (CollectionUtils.isEmpty(projectIds)) {
            return projectCostItems;
        }
        // 统计项目总的商业面积
        Double totalCommercialArea = saasWebService.getTotalCommercialArea(projectIds);

        for (String projectId : projectIds) {
            // 获取项目信息
            ReportProjectVO reportProjectInfo = saasWebService.getReportProjectInfo(projectId);
            if (reportProjectInfo == null) {
                continue;
            }
            CostItemVO costItemVO = getCostItemInfo(reportMonth, reportProjectInfo, totalCommercialArea, function.getId());
            projectCostItems.add(costItemVO);
        }
        return projectCostItems;
    }

    @Override
    public CostItemVO getCostItemInfo(
            Date reportMonth, ReportProjectVO reportProjectInfo,
            Double totalCommercialArea, String functionId) {
        // 费用展示为万元，定义一万元变量
        Double tenThousand = 10000d;
        String projectId = reportProjectInfo.getProjectId();
        CostItemVO costItemVO = new CostItemVO();
        String currentMonthStr = functionValueService.getFunctionValue(functionId, reportMonth, projectId);
        Double currentMonth = null;

        if (StringUtils.isNotBlank(currentMonthStr)) {
            currentMonth = Double.valueOf(currentMonthStr);
            costItemVO.setCurrentMonth(DataUtils.doubleDivide(currentMonth, tenThousand));
        }
        String lastMonthStr = functionValueService.getFunctionValue(
                functionId, DateUtils.getFirstDayOfLastMonth(reportMonth), projectId);
        Double lastMonth = null;
        if (StringUtils.isNotBlank(lastMonthStr)) {
            lastMonth = Double.valueOf(lastMonthStr);
            costItemVO.setLastMonth(DataUtils.doubleDivide(lastMonth, tenThousand));
        }

        // 统计去年同期的费用
        Double lastYearCount = null;
        List<Date> lastYearMonths = DateUtils.getFirstDayOfEveryMonth(
                DateUtils.getFirstDayOfLastYear(reportMonth),
                DateUtils.getSameMonthFirstDayOfLastYear(reportMonth));
        List<CostItemVO> lastYearItems = new ArrayList<>();
        for (Date lastYearMonth : lastYearMonths) {
            CostItemVO tmp = new CostItemVO();
            String monthValue = functionValueService.getFunctionValue(functionId, lastYearMonth, projectId);
            if (StringUtils.isNotBlank(monthValue)) {
                tmp.setLastYearCount(Double.parseDouble(monthValue));
                lastYearItems.add(tmp);
            }
        }
        Double zero = 0d;
        CostItemVO lastYearSumItem = getCostSum(lastYearItems);
        if (lastYearSumItem != null) {
            lastYearCount = lastYearSumItem.getLastYearCount();
            if (lastYearCount != null) {
                costItemVO.setLastYearCount(DataUtils.doubleDivide(lastYearCount, tenThousand));
            }
            if (totalCommercialArea != null && !totalCommercialArea.equals(zero) && lastYearCount != null) {
                costItemVO.setLastYearPerCount(DataUtils.doubleDivide(lastYearCount, totalCommercialArea));
            }
        }

        // 统计今年的费用
        Double yearCount = null;
        List<Date> yearMonths = DateUtils.getFirstDayOfEveryMonth(
                DateUtils.getFirstDayOfYear(reportMonth),
                reportMonth);
        List<CostItemVO> yearItems = new ArrayList<>();
        for (Date yearMonth : yearMonths) {
            CostItemVO tmp = new CostItemVO();
            String monthValue = functionValueService.getFunctionValue(functionId, yearMonth, projectId);
            if (StringUtils.isNotBlank(monthValue)) {
                tmp.setYearCount(Double.parseDouble(monthValue));
                yearItems.add(tmp);
            }
        }

        CostItemVO yearSumItem = getCostSum(yearItems);
        if (yearSumItem != null) {
            yearCount = yearSumItem.getYearCount();
            if (yearCount != null) {
                costItemVO.setYearCount(DataUtils.doubleDivide(yearCount, tenThousand));
            }
            if (totalCommercialArea != null && !totalCommercialArea.equals(zero) && yearCount != null) {
                costItemVO.setYearPerCount(DataUtils.doubleDivide(yearCount, totalCommercialArea));
            }
        }

        if (currentMonth != null && lastMonth != null) {
            Double linkCount = DataUtils.doubleSubtract(currentMonth, lastMonth);
            Double linkRange = DataUtils.doubleDivide(linkCount, lastMonth);
            costItemVO.setLinkCount(linkCount);
            costItemVO.setLinkRange(linkRange);
        }

        if (lastYearCount != null && yearCount != null) {
            double sameCount = DataUtils.doubleSubtract(yearCount, lastYearCount);
            costItemVO.setSameCount(sameCount);
            costItemVO.setSameRange(DataUtils.doubleDivide(sameCount, lastYearCount));
        }
        return costItemVO;
    }


    @Override
    public CostItemVO getCostSum(List<CostItemVO> costItems) {
        return DataUtils.sumDoubleParams(costItems, CostItemVO.class);
    }


    @Override
    public List<GroupInfo<CostItemVO>> getCostGroupInfos(List<String> groupIdList, Date reportMonth, List<String> projectIds) {
        List<GroupInfo<CostItemVO>> groupInfoList = Lists.newArrayList();
        if (CollectionUtils.isEmpty(groupIdList)) {
            return groupInfoList;
        }
        for (String groupId : groupIdList) {
            GroupInfo<CostItemVO> groupInfo = new GroupInfo<>();
            FunctionGroup functionGroup = functionGroupService.queryFunctionGroupDetail(groupId);
            groupInfo.setGroupName(functionGroup.getName());
            // 获取组下的分组（本期不做）
            // 获取组下的分项
            String functionIdStr = functionGroup.getFunctionIds();
            List<String> functionIds = Lists.newArrayList(functionIdStr.split(","));
            List<CostItemVO> costItemInfos = getCostItemInfos(functionIds, reportMonth, projectIds);
            groupInfo.setItems(costItemInfos);
            groupInfo.setSummary(getCostSum(costItemInfos));
            groupInfoList.add(groupInfo);
        }
        return groupInfoList;
    }
}
